@@ -14,6 +14,7 @@
 
 #include "bool.h"
 #include "error.h"
+#include "gvars.h"
 #include "integer.h"
 #include "io.h"
 #include "lists.h"
@@ -104,8 +105,8 @@ static Obj FuncApplyRel(Obj self,
     /*T 1996/12/03 fceller this should be replaced by 'PlistConv'          */
     RequirePlainList(0, app);
     if ( LEN_PLIST(app) != 4 ) {
-        ErrorQuit( "<app> must be a list of length 4 not %d",
-                   (Int) LEN_PLIST(app), 0L );
+        ErrorQuit("<app> must be a list of length 4 not %d",
+                  (Int)LEN_PLIST(app), 0);
     }
 
     /* get the four entries                                                */
@@ -171,7 +172,7 @@ static void CompressDeductionList ( void )
 
     /* check if the situation is as assumed                                */
     if ( dedlst != dedSize ) {
-        ErrorQuit( "invalid call of CompressDeductionList", 0L, 0L );
+        ErrorQuit("invalid call of CompressDeductionList", 0, 0);
     }
 
     /* run through the lists and compress them                             */
@@ -194,7 +195,7 @@ static void CompressDeductionList ( void )
     /* check if we have at least one free position                         */
     if ( dedlst == dedSize ) {
         if ( dedprint == 0 ) {
-            Pr( "#I  WARNING: deductions being discarded\n", 0L, 0L );
+            Pr("#I  WARNING: deductions being discarded\n", 0, 0);
             dedprint = 1;
         }
         dedlst--;
@@ -580,7 +581,7 @@ static Obj FuncMakeConsequencesPres(Obj self, Obj list)
     /* check the definitions lists                                         */
     if ( ! ( IS_PLIST(objDefs1) && IS_PLIST(objDefs2) &&
         LEN_PLIST(objDefs1) == LEN_PLIST(objDefs2) ) ) {
-        ErrorQuit( "inconsistent definitions lists", 0L, 0L );
+        ErrorQuit("inconsistent definitions lists", 0, 0);
     }
     ndefsMax = LEN_PLIST(objDefs1);
     apply = 1;
@@ -623,7 +624,7 @@ static Obj FuncMakeConsequencesPres(Obj self, Obj list)
                 }
                 ndefs++;
                 if ( ndefs > ndefsMax ) {
-                    ErrorQuit( "inconsistent definitions lists", 0L, 0L );
+                    ErrorQuit("inconsistent definitions lists", 0, 0);
                 }
                 SET_ELM_PLIST( objDefs1, ndefs, INTOBJ_INT( lc ) );
                 SET_ELM_PLIST( objDefs2, ndefs, ptNums[lp] );
@@ -684,7 +685,7 @@ static Obj FuncStandardizeTableC(Obj self, Obj table, Obj stan)
                 (Int)TNAM_OBJ(ptTable[j]) );
         }
     }
-    if ( IS_INTOBJ(stan) && INT_INTOBJ(stan) == 1 ) {
+    if (stan == INTOBJ_INT(1)) {
        /* use semilenlex standard                                          */
        nloop = nrgen;
     }
@@ -751,7 +752,6 @@ static Obj FuncStandardizeTableC(Obj self, Obj table, Obj stan)
     /* clean out  */
     CleanOut();
 
-    /* return void                                                         */
     return 0;
 }
 
@@ -1034,7 +1034,7 @@ static Int AddCosetFactor2 (
                     /* used to be unrecoverable error message: 
                     ErrorQuit(
                         "exponent too large, Modified Todd-Coxeter aborted",
-                        0L, 0L ); */
+                        0, 0); */
                 }
                 ptWord[i] = sum;
             }
@@ -1050,7 +1050,7 @@ static Int AddCosetFactor2 (
                     /* used to be unrecoverable error message: 
                     ErrorQuit(
                         "exponent too large, Modified Todd-Coxeter aborted",
-                        0L, 0L ); */
+                        0, 0); */
                 }
                 ptWord[i] = sum;
             }
@@ -1119,8 +1119,8 @@ static Obj FuncApplyRel2(Obj self, Obj app, Obj rel, Obj nums)
     /* get and check the application list                                  */
     RequirePlainList(0, app);
     if ( LEN_PLIST(app) != 9 ) {
-        ErrorQuit( "<app> must be a list of length 9 not %d",
-                   (Int) LEN_PLIST(app), 0L );
+        ErrorQuit("<app> must be a list of length 9 not %d",
+                  (Int)LEN_PLIST(app), 0);
     }
     ptApp = BASE_PTR_PLIST(app) - 1;
 
@@ -1200,7 +1200,7 @@ static Obj FuncApplyRel2(Obj self, Obj app, Obj rel, Obj nums)
             ptTree = BASE_PTR_PLIST(objTree) - 1;
             treeWordLength = INT_INTOBJ( ptTree[4] );
             if ( LEN_PLIST(objTree2) != treeWordLength ) {
-                ErrorQuit( "ApplyRel2: illegal word length", 0L, 0L );
+                ErrorQuit("ApplyRel2: illegal word length", 0, 0);
             }
 
             /* initialize the coset representative word                    */
@@ -1247,7 +1247,7 @@ static Obj FuncApplyRel2(Obj self, Obj app, Obj rel, Obj nums)
             /* copy the result to its destination, if necessary            */
             if ( ptWord != ptTree2 ) {
                 if ( LEN_PLIST(word) != treeWordLength ) {
-                    ErrorQuit( "illegal word length", 0L, 0L );
+                    ErrorQuit("illegal word length", 0, 0);
                 }
                 for ( i = 1;  i <= treeWordLength;  i++ ) {
                     ptWord[i] = ptTree2[i];
@@ -1352,7 +1352,6 @@ static Obj FuncCopyRel(Obj self, Obj rel) /* the given relator */
     Obj *               ptCopy;         /* pointer to the copy             */
     Int                 leng;           /* length of the given word        */
 
-    /* Get and check argument                                              */
     RequirePlainList(0, rel);
     leng = LEN_PLIST(rel);
 
@@ -1390,7 +1389,6 @@ static Obj FuncMakeCanonical(Obj self, Obj rel) /* the given relator */
     Int                 i, j, k, l;     /* integer variables               */
     Int                 ii, jj, kk;     /* integer variables               */
 
-    /* Get and check the argument                                          */
     RequirePlainList(0, rel);
     leng  = LEN_PLIST(rel);
     if (leng == 0) {
@@ -1522,7 +1520,6 @@ static Obj FuncMakeCanonical(Obj self, Obj rel) /* the given relator */
         }
     }
 
-    /* return nothing                                                      */
     return 0;
 }
 
@@ -1557,17 +1554,17 @@ static Obj FuncTreeEntry(Obj self, Obj tree, Obj word)
     /*  Get and check the first argument (tree)                            */
     objTree = tree;
     if ( ! IS_PLIST(tree) || LEN_PLIST(tree) < 5 ) {
-        ErrorQuit( "invalid <tree>", 0L, 0L );
+        ErrorQuit("invalid <tree>", 0, 0);
     }
 
     /*  Get and check the tree components                                  */
     objTree1 = ELM_PLIST(objTree,1);
     if ( ! IS_PLIST(objTree1) ) {
-        ErrorQuit( "invalid <tree>[1]", 0L, 0L );
+        ErrorQuit("invalid <tree>[1]", 0, 0);
     }
     objTree2 = ELM_PLIST(objTree,2);
     if ( ! IS_PLIST(objTree2) ) {
-        ErrorQuit( "invalid <tree>[2]", 0L, 0L );
+        ErrorQuit("invalid <tree>[2]", 0, 0);
     }
     ptTree1 = BASE_PTR_PLIST(objTree1) - 1;
     ptTree2 = BASE_PTR_PLIST(objTree2) - 1;
@@ -1578,14 +1575,14 @@ static Obj FuncTreeEntry(Obj self, Obj tree, Obj word)
 
     /*  Get the second argument (word)                                     */
     if ( ! IS_PLIST(word) ) {
-        ErrorQuit( "invalid <word>", 0L, 0L );
+        ErrorQuit("invalid <word>", 0, 0);
     }
 
     /* handle the abelianized case                                         */
     ptWord = BASE_PTR_PLIST(word) - 1;
     if ( treeType == 0 ) {
         if ( LEN_PLIST(word) != treeWordLength ) {
-            ErrorQuit( "inconsistent <word> length", 0L, 0L );
+            ErrorQuit("inconsistent <word> length", 0, 0);
         }
         ptWord = BASE_PTR_PLIST(objTree2) - 1;
         for ( leng = treeWordLength;  leng >= 1;  leng-- ) {
@@ -1654,14 +1651,14 @@ static Obj FuncTreeEntry(Obj self, Obj tree, Obj word)
 
     /* handle the general case                                             */
     if ( LEN_PLIST(objTree1) != LEN_PLIST(objTree2) ) {
-        ErrorQuit( "inconsistent <tree> components", 0L, 0L );
+        ErrorQuit("inconsistent <tree> components", 0, 0);
     }
 
     for ( i = 1;  i <= numgens;  i++ ) {
         if ( INT_INTOBJ(ptTree1[i]) <= -i || INT_INTOBJ(ptTree1[i]) >= i
           || INT_INTOBJ(ptTree2[i]) <= -i || INT_INTOBJ(ptTree2[i]) >= i )
         {
-            ErrorQuit( "invalid <tree> components", 0L, 0L );
+            ErrorQuit("invalid <tree> components", 0, 0);
         }
     }
 
@@ -1673,7 +1670,7 @@ static Obj FuncTreeEntry(Obj self, Obj tree, Obj word)
             continue;
         }
         if ( gen > numgens || gen < -numgens ) {
-            ErrorQuit( "invalid <word> entry [%d]", i, 0L );
+            ErrorQuit("invalid <word> entry [%d]", i, 0);
         }
         if ( j > 0 && gen == - INT_INTOBJ(ptWord[j]) ) {
             j--;
@@ -1828,7 +1825,7 @@ static Obj FuncStandardizeTable2C(Obj self, Obj table, Obj table2, Obj stan)
     }
     objTable2 = table2;
     ptTabl2 = BASE_PTR_PLIST(objTable2) - 1;
-    if ( IS_INTOBJ(stan) && INT_INTOBJ(stan) == 1 ) {
+    if (stan == INTOBJ_INT(1)) {
        /* use semilenlex standard                                          */
        nloop = nrgen;
     }
@@ -1902,7 +1899,6 @@ static Obj FuncStandardizeTable2C(Obj self, Obj table, Obj table2, Obj stan)
         SET_LEN_PLIST( ptTabl2[2*j  ], lcos );
     }
 
-    /* return void                                                         */
     return 0;
 }
 
@@ -1925,22 +1921,17 @@ static Obj FuncAddAbelianRelator(Obj self,
     Int                 numrows;        /* number of relators              */
     Int                 i, j;           /* loop variables                  */
 
-    /* check the arguments                                                 */
     RequirePlainList(0, rels);
     ptRels = BASE_PTR_PLIST(rels) - 1;
-    if ( !IS_INTOBJ(number) ) {
-        ErrorQuit( "<number> must be a small integer (not a %s)",
-            (Int)TNAM_OBJ(number), 0L );
-    }
 
     /* get the length of the given relators list                           */
-    numrows = INT_INTOBJ(number);
+    numrows = GetPositiveSmallInt(SELF_NAME, number);
     if ( numrows < 1 || LEN_PLIST(rels) < numrows ) {
-        ErrorQuit( "inconsistent relator number", 0L, 0L );
+        ErrorQuit("inconsistent relator number", 0, 0);
     }
     tmp = ELM_PLIST( rels, numrows );
     if ( tmp == 0 ) {
-        ErrorQuit( "inconsistent relator number", 0L, 0L );
+        ErrorQuit("inconsistent relator number", 0, 0);
     }
     pt2 = BASE_PTR_PLIST(tmp) - 1;
 
@@ -2232,6 +2223,19 @@ static Obj FuncLOWINDEX_PREPARE_RELS(Obj self, Obj r) /* rels */
    return (Obj) 0;
 }
 
+/****************************************************************************
+**
+*F  FuncNEW_LOWINDEX_DATA( <n> )
+**
+*/
+static Obj FuncNEW_LOWINDEX_DATA(Obj self, Obj n)
+{
+    Int len = GetSmallInt(SELF_NAME, n);
+    Obj rel = NewBag(T_DATOBJ, (len + 1) * sizeof(Obj));
+    SET_TYPE_DATOBJ(rel, TYPE_LOWINDEX_DATA);
+    return rel;
+}
+
 /****************************************************************************
 **
 *F  FuncTC_QUICK_SCAN( <c>,<o>,<alpha>,<w>)
@@ -2304,20 +2308,21 @@ static Obj FuncTC_QUICK_SCAN(Obj self,
 */
 static StructGVarFunc GVarFuncs [] = {
 
-    GVAR_FUNC(ApplyRel, 2, "app, relator"),
-    GVAR_FUNC(MakeConsequences, 1, "list"),
-    GVAR_FUNC(MakeConsequencesPres, 1, "list"),
-    GVAR_FUNC(StandardizeTableC, 2, "table, standard"),
-    GVAR_FUNC(ApplyRel2, 3, "app, relators, nums"),
-    GVAR_FUNC(CopyRel, 1, "relator"),
-    GVAR_FUNC(MakeCanonical, 1, "relator"),
-    GVAR_FUNC(TreeEntry, 2, "relator, word"),
-    GVAR_FUNC(StandardizeTable2C, 3, "table, table, standard"),
-    GVAR_FUNC(AddAbelianRelator, 2, "rels, number"),
-    GVAR_FUNC(LOWINDEX_COSET_SCAN, 4, "table, relators, stack1,stack2"),
-    GVAR_FUNC(LOWINDEX_IS_FIRST, 4, "table, n, mu, nu"),
-    GVAR_FUNC(LOWINDEX_PREPARE_RELS, 1, "rels"),
-    GVAR_FUNC(TC_QUICK_SCAN, 5, "table, offset, alpha, word, result"),
+    GVAR_FUNC_2ARGS(ApplyRel, app, relator),
+    GVAR_FUNC_1ARGS(MakeConsequences, list),
+    GVAR_FUNC_1ARGS(MakeConsequencesPres, list),
+    GVAR_FUNC_2ARGS(StandardizeTableC, table, standard),
+    GVAR_FUNC_3ARGS(ApplyRel2, app, relators, nums),
+    GVAR_FUNC_1ARGS(CopyRel, relator),
+    GVAR_FUNC_1ARGS(MakeCanonical, relator),
+    GVAR_FUNC_2ARGS(TreeEntry, relator, word),
+    GVAR_FUNC_3ARGS(StandardizeTable2C, table, table, standard),
+    GVAR_FUNC_2ARGS(AddAbelianRelator, rels, number),
+    GVAR_FUNC_4ARGS(LOWINDEX_COSET_SCAN, table, relators, stack1, stack2),
+    GVAR_FUNC_4ARGS(LOWINDEX_IS_FIRST, table, n, mu, nu),
+    GVAR_FUNC_1ARGS(LOWINDEX_PREPARE_RELS, rels),
+    GVAR_FUNC_1ARGS(NEW_LOWINDEX_DATA, n),
+    GVAR_FUNC_5ARGS(TC_QUICK_SCAN, table, offset, alpha, word, result),
     { 0, 0, 0, 0, 0 }
 
 };
@@ -2333,8 +2338,8 @@ static Int InitKernel (
     /* init filters and functions                                          */
     InitHdlrFuncsFromTable( GVarFuncs );
 
-    /* import kind (and unkind) functions */
-    ImportGVarFromLibrary( "TYPE_LOWINDEX_DATA",&TYPE_LOWINDEX_DATA     );
+    // import type object
+    InitCopyGVar("TYPE_LOWINDEX_DATA", &TYPE_LOWINDEX_DATA);
 
     /* static variables                                                    */
     InitGlobalBag( &objRel      , "src/costab.c:objRel"       );
@@ -2350,7 +2355,6 @@ static Int InitKernel (
     InitGlobalBag( &objWordValue, "src/costab.c:objWordValue" );
     InitGlobalBag( &objExponent , "src/costab.c:objExponent"  );
 
-    /* return success                                                      */
     return 0;
 }
 
@@ -2365,7 +2369,6 @@ static Int InitLibrary (
     /* init filters and functions                                          */
     InitGVarFuncsFromTable( GVarFuncs );
 
-    /* return success                                                      */
     return 0;
 }
 
