@@ -9,7 +9,6 @@
 **
 **  This file declares functions and variables related to command line
 **  settable options.
-**
 */
 
 #ifndef GAP_SYSOPT_H
@@ -22,13 +21,6 @@
 *F * * * * * * * * * * * command line settable options  * * * * * * * * * * *
 */
 
-
-/****************************************************************************
-**
-*V  SyArchitecture  . . . . . . . . . . . . . . . .  name of the architecture
-*/
-extern const Char * SyArchitecture;
-
 /****************************************************************************
 **
 *V  SyCTRD  . . . . . . . . . . . . . . . . . . .  true if '<ctr>-D' is <eof>
@@ -36,39 +28,20 @@ extern const Char * SyArchitecture;
 extern UInt SyCTRD;
 
 
-/****************************************************************************
-**
-*V  SyCompileInput  . . . . . . . . . . . . . . . . . .  from this input file
-*/
-extern Char SyCompileInput[GAP_PATH_MAX];
-
-
-/****************************************************************************
-**
-*V  SyCompileMagic1 . . . . . . . . . . . . . . . . . . and this magic number
-*/
-extern Char * SyCompileMagic1;
-
-
-/****************************************************************************
-**
-*V  SyCompileName . . . . . . . . . . . . . . . . . . . . . .  with this name
-*/
-extern Char SyCompileName[256];
-
-
-/****************************************************************************
-**
-*V  SyCompileOutput . . . . . . . . . . . . . . . . . . into this output file
-*/
-extern Char SyCompileOutput[GAP_PATH_MAX];
-
-
 /****************************************************************************
 **
 *V  SyCompilePlease . . . . . . . . . . . . . . .  tell GAP to compile a file
+*V  SyCompileOutput . . . . . . . . . . . . . . . . . . into this output file
+*V  SyCompileInput  . . . . . . . . . . . . . . . . . .  from this input file
+*V  SyCompileName . . . . . . . . . . . . . . . . . . . . . .  with this name
+*V  SyCompileMagic1 . . . . . . . . . . . . . . . . . . and this magic string
 */
 extern Int SyCompilePlease;
+extern Char * SyCompileOutput;
+extern Char * SyCompileInput;
+extern Char * SyCompileName;
+extern Char * SyCompileMagic1;
+
 
 /****************************************************************************
 **
@@ -78,28 +51,9 @@ extern Int SyDebugLoading;
 
 /****************************************************************************
 **
-*V  SyGapRootPaths  . . . . . . . . . . . . . . . . . . . array of root paths
-**
-**  'SyGapRootPaths' conatins the  names   of the directories where   the GAP
-**  files are located.
-**
-**  It is modified by the command line option -l.
-**
-**  It is copied into the GAP variable 'GAPInfo.RootPaths' and used by
-**  'SyFindGapRootFile'.
-**
-**  Each entry must end  with the pathname seperator, eg.  if 'init.g' is the
-**  name of a library file 'strcat( SyGapRootPaths[i], "lib/init.g" );'  must
-**  be a valid filename.
-**
-**  In addition we store the path to the users ~/.gap directory, if available,
-**  in 'DotGapPath'.
-**
-**  Put in this package because the command line processing takes place here.
+*F  SyDotGapPath()
 */
-enum { MAX_GAP_DIRS = 16 };
-extern Char SyGapRootPaths[MAX_GAP_DIRS][GAP_PATH_MAX];
-extern Char DotGapPath[GAP_PATH_MAX];
+const Char * SyDotGapPath(void);
 
 /****************************************************************************
 **
@@ -119,25 +73,6 @@ extern UInt SyLineEdit;
 */
 extern UInt SyUseReadline;
 
-/****************************************************************************
-**
-*V  SyMsgsFlagBags  . . . . . . . . . . . . . . . . .  enable gasman messages
-**
-**  'SyMsgsFlagBags' determines whether garabage collections are reported  or
-**  not.
-**
-**  Per default it is false, i.e. Gasman is silent about garbage collections.
-**  It can be changed by using the  '-g'  option  on the  GAP  command  line.
-**
-**  This is used in the function 'SyMsgsBags' below.
-**
-**  Put in this package because the command line processing takes place here.
-*/
-extern UInt SyMsgsFlagBags;
-
-
-extern Int SyGasmanNumbers[2][9];
-
 /****************************************************************************
 **
 *V  SyNrCols  . . . . . . . . . . . . . . . . . .  length of the output lines
@@ -207,7 +142,9 @@ extern UInt SyQuitOnBreak;
 **  of a workspace to restore.
 **
 */
+#ifdef GAP_ENABLE_SAVELOAD
 extern Char * SyRestoring;
+#endif
 
 /****************************************************************************
 **
