@@ -10,7 +10,6 @@ gap="$GAPDIR/bin/gap.sh"
 gac="$GAPDIR/gac"
 for gfile in *.g; do
     echo "Regenerating ${gfile}.out ..."
-    ./run_interpreted.sh "${gap}" "${gfile}" > "${gfile}.out"
+    ./run_interpreted.sh "${gap}" "${gfile}" "${gfile}.out"
     "${gac}" -d -C -o "${gfile}.dynamic.c" "${gfile}"
-    "${gac}" -C -o "${gfile}.static.c" "${gfile}"
 done
